#include <windows.h>
extern "C" {
#include "fitz.h"
#include "mupdf.h"
}
#define XMD_H
#define HAVE_BOOLEAN
#define NEED_FAR_POINTERS
#include "jpeglib.h"
#undef XMD_H
#undef HAVE_BOOLEAN
#undef NEED_FAR_POINTERS
#include <stdio.h>

int GetJpegToMemory(HLOCAL *hLocal, unsigned long *buf_size, pdf_xref *xref, pdf_page *page, int resolution)
{
	fz_display_list *list;
	fz_device *dev;
	fz_colorspace *colorspace = fz_device_rgb;
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;
	JSAMPARRAY img;

	cinfo.err = jpeg_std_error(&jerr);

	jpeg_create_compress(&cinfo);
	float zoom;
	fz_matrix ctm;
	fz_bbox bbox;
	fz_pixmap *pix;
	fz_glyph_cache_s *glyphcache;

	zoom = (float)resolution / 72;
	ctm = fz_translate(0, -page->mediabox.y1);
	ctm = fz_concat(ctm, fz_scale(zoom, -zoom));
	ctm = fz_concat(ctm, fz_rotate(page->rotate));
	bbox = fz_round_rect(fz_transform_rect(ctm, page->mediabox));

	pix = fz_new_pixmap_with_rect(colorspace, bbox);
	fz_clear_pixmap_with_color(pix, 255);

	glyphcache = fz_new_glyph_cache();
	dev = fz_new_draw_device(glyphcache, pix);
	pdf_run_page(xref, page, dev, ctm);
	fz_free_device(dev);

	fz_write_pnm(pix, "output.ppm");

	cinfo.image_width = pix->w;
	cinfo.image_height = pix->h;
	cinfo.input_components = 3;
	cinfo.in_color_space = JCS_RGB;
	jpeg_set_defaults(&cinfo);
	jpeg_set_quality(&cinfo, 100, TRUE);

	*buf_size = pix->w*pix->h*3;
	*hLocal = LocalAlloc(LMEM_FIXED, *buf_size);
	unsigned char * buf = (unsigned char *)*hLocal;

	jpeg_mem_dest(&cinfo, &buf, buf_size);
	jpeg_start_compress(&cinfo, TRUE);

	JSAMPROW row_pointer[1];
	JSAMPROW row;
	row = (JSAMPROW)malloc(sizeof(JSAMPLE)*pix->w*3);
	if (row == NULL) {
		return 0;
	}
	unsigned char *ps = pix->samples, *pd;
	for (int i=0;i<pix->h;i++) {
		pd = row;
		for (int j=0;j<pix->w;j++) {
			*(pd++) = *(ps++);
			*(pd++) = *(ps++);
			*(pd++) = *(ps++);
			ps++;
		}
		row_pointer[0] = row;
		jpeg_write_scanlines(&cinfo, row_pointer, 1);
	}

	jpeg_finish_compress(&cinfo);
	free(row);
	fz_free_glyph_cache(glyphcache);
	fz_drop_pixmap(pix);
	jpeg_destroy_compress(&cinfo);

	return 0;
}

int main(int argc, char *argv[])
{
	pdf_xref *xref;
	pdf_page *page;
	fz_error error;
	int pagenum = 0;
	int resolution = 72;//dpi
	HLOCAL hLocal;
	unsigned long filesize;

	error = pdf_open_xref(&xref, "b.pdf", NULL);
	if (error) {
		printf("open error\n");
		return 0;
	}
	error = pdf_load_page_tree(xref);
	if (error) {
		printf("tree error\n");
		return 0;
	}
	printf("%d", pdf_count_pages(xref));
	error = pdf_load_page(&page, xref, pagenum);
	if (error) {
		printf("load page error\n");
		return 0;
	}
	GetJpegToMemory(&hLocal, &filesize, xref, page, resolution);

	unsigned char *p = (unsigned char *)hLocal;
	FILE *fp = fopen("output.jpg", "wb");
	for (int i=0;i<filesize;i++) {
		putc(p[i], fp);
	}
	fclose(fp);

	LocalFree(hLocal);
	
	pdf_free_page(page);
	pdf_free_xref(xref);
	return 0;
}
